#include <camcapture.h>
#include <stdio.h>
#include <stdlib.h>
#include "debugdisplay.h"


void draw_debug(image_data_t *img, int frame, int edges)
{
	color_rgb_t c;
	char buf[512];
	c.r = 0; c.g = 0; c.b = 200;
	sprintf(buf, "frame: %d", frame);
	camcapture_draw_string(img, 10, 430, buf, &c);
	sprintf(buf, "edges: %d", edges);
	camcapture_draw_string(img, 10, 440, buf, &c);
}



int main(int argc, char** argv)
{
	int width  = 800;
	int height = 600;
	const char *device = "/dev/video0";
	color_rgb_t red;
	red.r = 255; red.g = 0; red.b = 0;
	color_rgb_t green;
	green.r = 10; green.g = 255; green.b = 10;

	/* Debug-Fenster erzeugen */
	if( !(ddisplay_init(width, height)) )
		exit(-1);

	/* Init camcapture */
	if(!camcapture_init(device, width, height))
	{
		ddisplay_destruct();
		exit(-1);
	}

	/* Objekt für Bilddaten */
	image_data_t imgdata;
	image_data_t old;
	old.img = NULL;
	image_data_t diff;
	diff.img = NULL;

	/* Teste, ob programm weiter laufen soll */
	int frame = 1;
	while( ddisplay_running() )
	{
		/* Lock camera buffer */
		camcapture_lock();
		camcapture_get_frame(&imgdata);
		/* Erstelle kompatible bilder, wenn bilddaten noch null sind */
		if(old.img == NULL)  camcapture_create_compatible_image(&imgdata, &old);
		if(diff.img == NULL) camcapture_create_compatible_image(&imgdata, &diff);
		/* Berechne differenz */
		camcapture_filter_difference(&diff, &imgdata, &old, 40);
		camcapture_copy_image(&imgdata, &old);
		/* Lege listen für Kantenpunkte und Flächenpunkte an */
		image_point_list_t *edgelist;
		image_point_list_t *rectlist;
		edgelist = camcapture_create_point_list();
		rectlist = camcapture_create_point_list();
		/* Suche kanten, suche Rechtecke, die Threshold übersteigen */
		int found = camcapture_edge_detection_x(&diff, 40, 9, 2048, edgelist);
		camcapture_rect_threshold(&diff, 100, 30, rectlist);
		/* Zeichnen */
		camcapture_draw_point_list_crossed(&diff, edgelist, &green);
		camcapture_draw_point_list_rects(&diff, rectlist, 30, &red);
		/* Aufräumen */
		camcapture_destroy_point_list(edgelist);
		camcapture_destroy_point_list(rectlist);
		/* Infotext */
		draw_debug(&diff, frame, found);
		/* Anzeigen */
		ddisplay_show(&diff);
		camcapture_unlock();
		frame++;
	}

	/* Deinitialisiere debugdisplay */
	ddisplay_destruct();

	return 0;
}
